/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

using namespace std;

File::File(char *filename)
{
	MakeString(this->filename, filename);
}
     
File::~File(void)
{
        if (this->filename)
                FreeString(this->filename);
}
                        
void File::Load(void)
{
	char x;
	char data[256];
	int i = 0;

	errno = 0;

	this->stream.open(this->filename, ios::in | ios::binary);

	if(errno) {
#ifdef SDEBUG
		DebugLog->Write("Error opening %s: %s", this->filename, strerror(errno));
#endif
		errno = 0;
		return;
	}

	this->stream.seekg(0, ios::beg);
	for(;;) {
		this->stream.read((char *) (&x), sizeof(x));
		if(this->stream.eof()) break;
		if(x == '\n' || x == '\r') {
			data[i] = '\0';
			this->Parse((char *) data);
			i = 0;
		}
		else
			data[i++] = x;
	}
	data[i] = '\0';
	this->stream.close();
}

void File::Parse(char *data)
{
	char	*p, *q;
	char 	*args[32];
	int	c, argc, colon;
	char	*tempargs;

	p = q = data;
	c = argc = colon = 0;

	if(strlen(data) == 0)
		return;

	while (*p) {
		if(*p == ':') {
			colon = 1;
			q=(++p);
			continue;
		}
		else if(*p == ' ') {
			if(colon != 2) {
				args[argc] = new char[512];
				strncpy(args[argc], q, c);
				if(!argc)
					args[argc][c]='\0';
				else
					args[argc][c-1]='\0';
				c=0;
				if(!colon)
					argc++;
				q=(p+1);
				if(colon == 1)
					colon = 2;
			}
			else {
				tempargs = new char[512];
				strncpy(tempargs, q, c);
				tempargs[c-1] = '\0';
				c=0;
				q=(p+1);
				strcat(args[argc], " ");
				strcat(args[argc], tempargs);
				delete tempargs;
			}
		}
		p++;
		c++;
	} 

	if(colon != 2) {
		args[argc] = new char[512];
		sprintf(args[argc], q);
		argc++;
	}
	else {
		strcat(args[argc], " ");
		strcat(args[argc], q);
		argc++;
	}

	this->Process((char **)args, argc);

	for(c=0;c<argc;c++)
		delete args[c];
	
}

void File::Process(char **args, int argc)
{
	// Virtual function -GZ
}

void File::Replace(void)
{
	// Virtual function -GZ
}

void ChanDB::Process(char **args, int argc)
{
	aRegChan *reg;
	aRegNick *nick;

        if(!strcmp(args[0], "chandata1") && argc > 2) {  
	        if((reg = Registry->FindChannel(args[1]))) {
#ifdef SDEBUG
			DebugLog->Write("Found double entry in database for %s, ignoring.", args[1]);
#endif
                	return;
        	}
		Registry->AddChannel(args[1], args[2]);
        }
	else if(!strcmp(args[0], "chandata2") && argc > 2) {
	        if(!(reg = Registry->FindChannel(args[1]))) {
#ifdef SDEBUG
			DebugLog->Write("Found data entry for non-existing channel %s in database, ignoring.", args[1]);
#endif
                	return;
        	}
		Registry->AddChannelData1(reg, atol(args[2]));
	}
	else if(!strcmp(args[0], "chanop") && argc > 3) {
                if(!(reg = Registry->FindChannel(args[1]))) {
#ifdef SDEBUG
                        DebugLog->Write("Found data entry for non-existing channel %s in database, ignoring.", args[1]);
#endif  
                        return;
                }
                if(!(nick = Registry->FindNick(args[2]))) {
#ifdef SDEBUG
                        DebugLog->Write("Found data entry for unknown registered nick %s.", args[2]);
#endif  
                        return;
                }				
		Registry->AddChannelOp(reg, nick, atol(args[3]));
	}
}

void NickDB::Process(char **args, int argc)
{
        aRegNick *reg;
       
	/* This is where we add a new nick to the registry */

	if(!strcmp(args[0], "nickdata1") && argc > 4) {
	        if((reg = Registry->FindNick(args[1]))) {
#ifdef SDEBUG
			DebugLog->Write("Found double entry in database for %s, ignoring.", args[1]);
#endif
                	return;
        	}
		Registry->AddNick(args[1], args[2], args[3], args[4]);
	}

	/* This is where we will add registration data to an already
	 * existing entry in the Registry.
	 */

	else if(!strcmp(args[0], "nickdata2") && argc > 4) {
	        if(!(reg = Registry->FindNick(args[1]))) {
#ifdef SDEBUG
			DebugLog->Write("Found data entry for non-existing nick %s in database, ignoring.", args[1]);
#endif
                	return;
        	}
		Registry->AddNickData1(reg, args[2], args[3], atol(args[4]));
	}
}

void NickDB::Replace(void)
{
	char line[512];
	Link *c;
	RegLink *f;
	aRegNick *finddata;
 
	int	i;

	errno = 0;

	this->stream.open(this->filename, ios::out | ios::trunc | ios::binary);

        if(errno) {
#ifdef SDEBUG
                DebugLog->Write("Error writing to %s: %s", this->filename, strerror(errno));
#endif
		errno = 0;
                return;
	}

	this->stream.seekg(0, ios::beg);

	i = 0;

        for (c = Regdata; c; c = c->next) {
                f = ((RegLink*) c->value);
		if(!IsClient(f))
			continue;
                finddata = (((RegLink*) c->value)->nick);
	        sprintf(line, "nickdata1 %s %s %s :%s\n", f->name, finddata->username, finddata->hostname, finddata->realname);
	        this->stream.write(line, strlen(line));

		if(finddata->email && finddata->password) {
		        sprintf(line, "nickdata2 %s %s %s %lu\n", f->name, finddata->email, finddata->password, finddata->regflags);
		        this->stream.write(line, strlen(line));
		}
        }

	this->stream.close();
}

void ChanDB::Replace(void) {

	char line[512];
	Link *c, *j;
	RegLink *f;
	aRegChan *finddata;
	aChanOp *chanop;

	int	i;

	errno = 0;

	this->stream.open(this->filename, ios::out | ios::trunc | ios::binary);

        if(errno) {
#ifdef SDEBUG
                DebugLog->Write("Error writing to %s: %s", this->filename, strerror(errno));
#endif
		errno = 0;
                return;
	}

	this->stream.seekg(0, ios::beg);

	i = 0;

        for (c = Regdata; c; c = c->next) {
                f = ((RegLink*) c->value);
		if(!IsChannel(f))
			continue;
                finddata = (((RegLink*) c->value)->chan);
	        sprintf(line, "chandata1 %s %s\n", f->name, finddata->founder);
	        this->stream.write(line, strlen(line));

		sprintf(line, "chandata2 %s %lu\n", f->name, finddata->regflags);
		this->stream.write(line, strlen(line));

		for(j = finddata->oplist;j;j = j->next) {
			chanop = (aChanOp *) j->value;
			sprintf(line, "chanop %s %s %lu\n", finddata->name, chanop->name, chanop->level);
			this->stream.write(line, strlen(line));
		}
        }

	this->stream.close();

}
