/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.  
 
   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:      
 
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.  
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team
        
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/              

#include "services.h"

#include "language/english.h"
#include "language/dutch.h"

struct	MasterLangTable MLT[] = {
	{	 0, "English", 0, 0, NULL },
	{	 1, "Dutch", 0, 0, NULL },
	{	-1, NULL, 0, 0, NULL }
};

Language::Language(void)
{;}

Language::~Language(void)
{
	for(int i=0;i<MLT[0].numlangs;i++)
		delete MLT[i].pFormatTable;
}

void Language::LoadMasterTable(void)
{
	int	i;

	i = 0;

	while(MLT[i].langcode != -1) {
		MLT[i].numreplies = this->LoadTable(i);
		i++;
	}
	
	MLT[0].numlangs = i;
}

int Language::LoadTable(int TableCode)
{
	struct LangTable *pLangTable;

	int	i, j;

	i = j = 0;

	switch(TableCode) {
		case 0:
			pLangTable = EnglishReplies;
			break;
		case 1:
			pLangTable = DutchReplies;
			break;
		default:
			return -1;
	}

	MLT[TableCode].pFormatTable = new CompiledTable;

	while(pLangTable[i].numeric){
		MLT[TableCode].pFormatTable->Entry[pLangTable[i].numeric] = pLangTable[i].format;
		i++;
	}
	
#ifdef SDEBUG
	DebugLog->Write("Inserted %i numerics into language-table %i (%s)", i, TableCode, MLT[TableCode].langname);
#endif
	return i;
	
}

int Language::SetLanguage(User *uptr)
{
	char 	*hostname, *tld;
	char	*dotpos, *c;
	int	language = 0;

	if(IsMine(uptr))
		return 0;

	MakeString(hostname, uptr->hostname);

	dotpos = c = hostname;

	/* Get TLD from the hostname first to reduce searchtime -GZ */

	while(*c) {
		if(*c == '.')
			dotpos = c;
		c++;
	}

	MakeString(tld, dotpos);

	if(!strcasecmp(tld, ".COM") || !strcasecmp(tld, ".NET"))
		language = 0;
	else if (!strcasecmp(tld, ".NL"))
		language = 1;

	FreeString(hostname);
	FreeString(tld);

	return language;
}

char 	*Language::MakeReply(int numeric, User *uptr)
{
	int 	language = uptr->language;

	if(numeric > MAX_NUMERICS || numeric <= 0) {
#ifdef SDEBUG
		DebugLog->Write("Requested numeric %i not found!", numeric);
#endif
		numeric = 1;
	}

	if(MLT[language].pFormatTable->Entry[numeric] == NULL) {
		if(MLT[0].pFormatTable->Entry[numeric] == NULL)
			numeric = 1;
		else
			language = 0;
	}

	return MLT[language].pFormatTable->Entry[numeric];
}
