/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

/* List of IRC objects, meaning every single user/server/channel? */
Link		*Objects = NULL;
Link		*TimerList = NULL;
Link		*Regdata = NULL;

/* Link class object - Con/Destructor wrappers */
Link::Link(void *value, Link *prev, Link *next, Link *hprev, Link *hnext)
{
	this->Init(value, prev, next, hprev, hnext);
}

Link::Link(void *value)
{
	this->Init(value, NULL, NULL, NULL, NULL);
}

Link::Link(void *value, Link **whereto)
{
	this->Init(value, NULL, NULL, NULL, NULL);
	this->LinkTo(whereto);
}

Link::Link(void *value, Link **whereto, Link *hashtable[])
{
	this->Init(value, NULL, NULL, NULL, NULL);
	this->LinkTo(whereto, hashtable);
}

Link::~Link(void)
{
	this->Destruct();
}

/* Constructor:
   -------------
       Parameters:
	  value    = void pointer to object to be linked in
	  prev     = init value of prev-pointer (usually NULL)
	  next     = init value of prev-pointer          (-=-)
	  hprev    = init value of hash previous pointer (-=-)
	  hpnext   = init value of hash next pointer     (-=-)
*/
void Link::Init(void *value)
{
	this->Init(value, NULL, NULL, NULL, NULL);
}

void Link::Init(void *value, Link **whereto)
{
	this->Init(value, NULL, NULL, NULL, NULL);
	this->LinkTo(whereto);
}

void Link::Init(void *value, Link **whereto, Link *hashtable[])
{
	this->Init(value, NULL, NULL, NULL, NULL);
	this->LinkTo(whereto, hashtable);
}
void Link::Init(void *value, Link *prev, Link *next, Link *hprev, Link *hnext)
{
	this->value = value;
	this->prev = prev;
	this->next = next;
	this->hprev = hprev;
	this->hnext = hnext;
}

/* Destructors, resets values to NULL */
void Link::Destruct(void)
{
	this->value = this->prev = this->next = this->hprev = this->hnext
		= NULL;
}


/* This method will link the Link together with a list, even an empty one
   
   Example:
	Link	*Objects = NULL;
	Link    *tmp = new Link(NickServ, NULL, NULL, NULL, NULL);
	
	tmp->LinkTo(&Objects);

   Notes:
	Must be called with the pointer to the pointer
         (&Objects)
*/

void	Link::LinkTo(Link **LinkList)
{
	if (!LinkList)
		return;	
	this->prev = NULL;
	this->next = *LinkList;
	if (this->next)
		this->next->prev = this; 
	*LinkList = this;
}

void	Link::LinkTo(Link **LinkList, Link *hashtable[])
{
	long	hash;
	User	*uptr;
	
	if (!LinkList)
		return;	
	if (!hashtable)
		return;
	this->prev = NULL;
	this->next = *LinkList;
	if (this->next)
		this->next->prev = this; 
	*LinkList = this;
	
	uptr = (User *) this->value;
	if (!uptr)
	{
		ServicesLog->Critical("Tried to link !uptr object to hashtable");
		return;
	}
	if (!uptr->name)
	{
		ServicesLog->Critical("Tried to link in !uptr->name to hashtable");
		return;
	}
	hash = get_hash_value(uptr->name);
	
	this->hprev = NULL;
	this->hnext = hashtable[hash];
	if (this->hnext)
		this->hnext->hprev = this;
	hashtable[hash] = this;
	 
}

void	Link::LinkRegTo(Link **LinkList, Link *hashtable[])
{
	long	hash;
	RegLink	*uptr;
	
	if (!LinkList)
		return;	
	if (!hashtable)
		return;
	this->prev = NULL;
	this->next = *LinkList;
	if (this->next)
		this->next->prev = this; 
	*LinkList = this;
	
	uptr = (RegLink *) this->value;
	if (!uptr)
	{
		ServicesLog->Critical("Tried to link !uptr object to hashtable");
		return;
	}
	if (!uptr->name)
	{
		ServicesLog->Critical("Tried to link in !uptr->name to hashtable");
		return;
	}
	hash = get_hash_value(uptr->name);
	
	this->hprev = NULL;
	this->hnext = hashtable[hash];
	if (this->hnext)
		this->hnext->hprev = this;
	hashtable[hash] = this;
	 
}

/* 
 * This will Unlink an Link from a list
 *
 * Example:
 *    tmp->Unlink(&Objects);
 * Notes:
 *    Must be called with the pointer to the list
 *    (&Objects)
*/
void	Link::Unlink(Link **LinkList)
{
	if (!LinkList)
		return;

	/* We even need a loop here?? --Stskeeps */
	if (this->prev)
		this->prev->next = this->next;
	else
		*LinkList = this->next; 
	
	if (this->next)
		this->next->prev = this->prev;			
}

void	Link::Unlink(Link **LinkList, Link *hashtable[])
{
	long	hash;
	User	*uptr;
	
	if (!LinkList)
		return;

	/* We even need a loop here?? --Stskeeps */
	if (this->prev)
		this->prev->next = this->next;
	else
		*LinkList = this->next; 

	if (this->next)
		this->next->prev = this->prev;			
	
	/* Hash stuff */
	uptr = (User *)this->value;	
	if (!uptr)
	{
#ifdef SDEBUG
		DebugLog->Write("LinkTo, !uptr");
#endif
		return;
	}
	if (!uptr->name)
	{
		ServicesLog->Critical("Tried to get null pointer hash");
		return;
	}
	hash = get_hash_value(uptr->name);

	if (this->hprev)
	{
		this->hprev->hnext = this->hnext;
	}
		else
	{
		hashtable[hash] = this->hnext;
	}
	if (this->hnext)
		this->hnext->hprev = this->hprev;
}
void	Link::RegUnlink(Link **LinkList, Link *hashtable[])
{
	long	hash;
	RegLink	*uptr;
	
	if (!LinkList)
		return;

	/* We even need a loop here?? --Stskeeps */
	if (this->prev)
		this->prev->next = this->next;
	else
		*LinkList = this->next; 

	if (this->next)
		this->next->prev = this->prev;			
	
	/* Hash stuff */
	uptr = (RegLink *)this->value;	
	if (!uptr)
	{
#ifdef SDEBUG
		DebugLog->Write("RegLinkTo, !uptr");
#endif
		return;
	}
	if (!uptr->name)
	{
		ServicesLog->Critical("Tried to get null pointer hash");
		return;
	}
	hash = get_hash_value(uptr->name);

	if (this->hprev)
	{
		this->hprev->hnext = this->hnext;
	}
		else
	{
		hashtable[hash] = this->hnext;
	}
	if (this->hnext)
		this->hnext->hprev = this->hprev;
}

void	Link::List(Link **LinkList)
{
	Link *p;
	Server *q;
	int i = 0;
	
	if (!LinkList)
		return;

	ServicesLog->Write("      Objects list    ");
        ServicesLog->Write("----------------------");

	for (p = *LinkList; p; p = p->next)
	{
		if (p->value)
		{
			q = (Server *) p->value;
			ServicesLog->Write("%-2i - %c%c%c%c%c - %s", i,
				(q->flags & FLAGS_CLIENT ? 'C' : ' '),
				(q->flags & FLAGS_SERVER ? 'S' : ' '),
				(q->flags & FLAGS_MINE   ? 'M' : ' '),
				(q->flags & FLAGS_CHANNEL ?'c' : ' '),
				(q->flags & FLAGS_MEMBER ? 'm' : ' '),
				q->name);
								
		}
		else
		{
			ServicesLog->Write("%i - NULL VALUE",i);
		}
		i++;
	}

}
