/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

/* This is the constructor, must have valid filename in "name" */
Log::Log(char *name)
{
	if (!name)
	{
		this->file = NULL;
		this->name = NULL;
		return;
	}
	this->file = fopen(name, "a");
	if (!this->file)
	{
		/* We should really output an error .. */
		return;
	}
	MakeString(this->name, name);
	this->flushcount = 1;
}

/* This will destruct the Log object, and closing the file */
Log::~Log(void)
{
	if (this->name)
		FreeString(this->name);
	if (this->file)
		fclose(this->file);

	this->name = NULL;
	this->file = NULL;
}

/* This will write to the log, in a printf alike format */
void	Log::Write(char *pattern, ...)
{
	va_list		ap;

        va_start(ap, pattern);
	va_end(ap);
	this->vWrite(pattern, ap);
}

void	Log::Critical(char *pattern, ...)
{
	va_list		ap;
	static char 	buf[512];

        va_start(ap, pattern);
        vsnprintf(buf, 512, pattern, ap);
	va_end(ap);
	if(ServicesConnect && MYNAME && (strlen(MYNAME) > 0))
#ifdef SDEBUG
		DebugLog->Write("Sending critical error: %s", buf);
#endif
		Connection.Send(":%s GLOBOPS :[\2CRITICAL\2] %s", MYNAME, buf);
	this->vWrite(pattern, ap);
	this->Flush();

}

/* Will flush the log buffers to disk */
void	Log::Flush(void)
{
	if (this->file)
		fflush(this->file);
}

/* vprintf alike log write */
void	Log::vWrite(char *pattern, va_list ap)
{
	static char 	buf[512], fmtime[80];
	time_t		tmp = time(NULL);

        vsnprintf(buf, 512, pattern, ap);
	strftime(fmtime, 80, "[%m/%d/%Y %H:%M]", localtime(&tmp));
	if (this->file)
	{
		fprintf(this->file, "%s %s\n", fmtime, buf);
#ifdef SDEBUG
		fprintf(stderr, "[%-10s] %s %s\n", this->name, fmtime, buf);
#endif
	}
	
	this->flushcount++;
	/* We flush auto every 5 hits */
	if (this->flushcount == 5)
	{
		this->Flush();
		this->flushcount = 0;
	}
}
