/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

Socket	Connection;
Parse	parse;
MessageTable msgtab;
Find	find;
Signal  *signals;
Timer	*Timers = NULL;

void	Loop(void);
void 	Init(void);
int	ReadConfiguration(void);
NSObject *NickServ = NULL;
CSObject *ChanServ = NULL;
Service	 *MemoServ = NULL;
OSObject *OperServ = NULL;

Log	*ServicesLog = NULL;
Mail	*Email = NULL;

#ifdef SDEBUG
Log	*DebugLog    = NULL;
#endif

Language Translate;

NickDB 	*NickServDB = NULL;
ChanDB  *ChanServDB = NULL;

Reg	*Registry = NULL;

char	CONNECTHOST[100];
int	CONNECTPORT;
char	CONNECTPASS[100];
char	MYNAME[100];
char    MYREALNAME[100];
char	MYDOMAIN[100];
char	NICKSERV[100];
char	CHANSERV[100];
char	MEMOSERV[100];
char	OPERSERV[100];
char	NICKSERVNAME[100];
char	CHANSERVNAME[100];
char	MEMOSERVNAME[100];
char	OPERSERVNAME[100];

bool	ServicesSafe = TRUE;
bool	ServicesConnect = FALSE;
bool	ServicesDBR = FALSE;

/* Main entrance */
int main(int argc, char *argv[])
{
	struct rlimit   corelim;
        corelim.rlim_cur = corelim.rlim_max = RLIM_INFINITY;
        setrlimit(RLIMIT_CORE, &corelim);

	ServicesLog = new Log("services.log");

#ifdef SDEBUG
	DebugLog = new Log("debug.log");
	DebugLog->Write("Loading Sirius IRC Services v%s...", VERSION);
#endif
	
	/* Initializing signal handlers */

	signals = new Signal(1);
	signals->Start();
#ifdef SDEBUG
	DebugLog->Write("Added signal handlers [mode %i].", signals->sigmode);
#endif

	/* Load our language tables */

	Translate.LoadMasterTable();

	/* Load our configuration */
	if (ReadConfiguration() < 0)
	{
		printf("Cannot load configuration, aborting\n");		
		exit(-1);
	}
	/* Connect us */
	if (Connection.Connect(CONNECTHOST, CONNECTPORT) < 0)
	{
#ifdef SDEBUG
		DebugLog->Write("Couldn't connect to server: %s", strerror(errno));
#endif
		exit(-1);
	} else {
		ServicesConnect = TRUE;
#ifdef SDEBUG
		DebugLog->Write("Connected to %s at port %i.", CONNECTHOST, CONNECTPORT);
#endif
	}

	Connection.Send("PASS %s", CONNECTPASS);
	Connection.Send("SERVER %s 1 :%s", MYNAME, MYREALNAME);
	Connection.Send("GLOBOPS :Sirius IRC Services v%s is now online", VERSION);
#ifdef SDEBUG
	DebugLog->Write("Clearing Objects and Regdata hashtable ...");
#endif
	/* Clearing hash */
	ObjectsHash_Reset();
	RegdataHash_Reset();
	Init();

	/* Intialize timers */
	Timers = new Timer();

	/* Initialize mail object */

	Email = new Mail();

	/* Determine network protocol */
#ifdef SDEBUG
	if(NETWORK_PROTOCOL == PROTOCOL_UNREAL)
		DebugLog->Write("Loaded Unreal IRCD protocols.");
	else if(NETWORK_PROTOCOL == PROTOCOL_STARCHAT)
		DebugLog->Write("Loaded StarChat IRCD protocols.");
#endif

	/* Call main loop */
	Loop();
}

void	Init(void)
{
	Link	*tmp;	

	/* Set me up .. */
	me = new Server(MYNAME, NULL);
	me->flags |= FLAGS_MINE;
	tmp = new Link(me, &Objects, ObjectsHash);

	/* Set up NickServ */		
	NickServ = new NSObject(NICKSERV, NICKSERV, MYDOMAIN, me, NICKSERVNAME, UMODE_OPER, 0);
	NickServ->flags |= FLAGS_MINE;
	tmp = new Link(NickServ, &Objects, ObjectsHash);

	/* Set up ChanServ */
	ChanServ = new CSObject(CHANSERV, CHANSERV, MYDOMAIN, me, CHANSERVNAME, UMODE_OPER, 0 );
	ChanServ->flags |= FLAGS_MINE;
	tmp = new Link(ChanServ, &Objects, ObjectsHash);

	/* Set up MemoServ */
	MemoServ = new Service(MEMOSERV, MEMOSERV, MYDOMAIN, me, MEMOSERVNAME, UMODE_OPER, 0 );
	MemoServ->flags |= FLAGS_MINE;
	tmp = new Link(MemoServ, &Objects, ObjectsHash);

	/* Set up OperServ */
	OperServ = new OSObject(OPERSERV, OPERSERV, MYDOMAIN, me, OPERSERVNAME, UMODE_OPER, 0 );
	OperServ->flags |= FLAGS_MINE;
	tmp = new Link(OperServ, &Objects, ObjectsHash);

	/* Spawn them */
	if(NickServ->Spawn())
		if(ChanServ->Spawn())
			if(MemoServ->Spawn())
				if(OperServ->Spawn()) 
#ifdef SDEBUG
					DebugLog->Write("Succesfully Spawned Services Agents.");
	else
					DebugLog->Write("Failed to spawn Services Agents.");
#else
	;
#endif

	/* Initialize registry */

	Registry = new Reg;

	char filename[512];

	/* Load nickname data */

	sprintf(filename,"%snickserv.db", DATABASEDIR);	
	NickServDB = new NickDB(filename);
	NickServDB->Load();

	/* Load channel data */

	sprintf(filename,"%schanserv.db", DATABASEDIR);	
	ChanServDB = new ChanDB(filename);
	ChanServDB->Load();

	/* Indicate that services databases are now as ready as they will ever get */

	ServicesDBR = TRUE;
}


/* This performs the general I/O loop to read/parse and check timers */

void	Loop(void)
{
	fd_set		readfds;
	struct timeval  tv;
	char		buffer[4096];
	int 		i = 0;
	int		error = 0;
	bzero(buffer, sizeof(buffer));

	tv.tv_sec = 0;
	tv.tv_usec = 200;

	for(;;) {
		
		Timers->Check();
		tv.tv_sec = 1;
		tv.tv_usec = 0;

		FD_ZERO(&readfds);
		FD_SET(Connection.fd, &readfds);
		error = select(Connection.fd + 1, &readfds, NULL, NULL, &tv);
		if (error > 0)		
		{
			if (FD_ISSET(Connection.fd, &readfds))
			{

				if(recv(Connection.fd, &buffer[i], 1, 0)) {
					if (buffer[i] == '\r' || buffer[i] == '\n') {
						buffer[i] = '\0';
						if (i != 0)
							parse.parse(buffer);
						i = 0;
						continue;
					}
					i++;
				}
			}
		}
		if (!error && errno != 0)
		{
			ServicesLog->Write("Connection closed: %s",
				strerror(errno));
			me->Shutdown(1);
			exit(0);
		}
	}
}


int	ReadConfiguration(void)
{
	int Result = 0, i = 0;

	char	*sections[] = 
	{
		"[Connection]",
		"[Agents]",
		(char *)NULL
	};	

	struct Config_Tag configs[] =
	{
		{"CONNECTHOST", String_Tag, CONNECTHOST},
		{"CONNECTPORT", DWord_Tag, &CONNECTPORT},
		{"CONNECTPASS", String_Tag, CONNECTPASS},
		{"MYNAME", String_Tag, MYNAME},
		{"MYREALNAME", String_Tag, MYREALNAME},
		{"MYDOMAIN", String_Tag, MYDOMAIN},
		{"NICKSERV", String_Tag, NICKSERV},
		{"NICKSERVNAME", String_Tag, NICKSERVNAME},
		{"CHANSERV", String_Tag, CHANSERV},
		{"CHANSERVNAME", String_Tag, CHANSERVNAME},
		{"MEMOSERV", String_Tag, MEMOSERV},
		{"MEMOSERVNAME", String_Tag, MEMOSERVNAME},
		{"OPERSERV", String_Tag, OPERSERV},
		{"OPERSERVNAME", String_Tag, OPERSERVNAME},
		{(char *)NULL, Error_Tag, (void *) NULL}
	};

	for (Result=0, i=0; sections[i]; i++)
        	Result += input_config (CONFIG_FILE, configs, sections[i]);
        if (Result == 0)
     	   ServicesLog->Critical("Configuration File is Empty.");
    	else if (Result < 0)
           ServicesLog->Critical("Configuration File not found.");
	return Result;
}
