/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

struct NMessage NSmsgtab[] = {
	{ "HELP", &NSObject::CmdHelp, 0, 0, 0, 0 },
	{ "IDENTIFY", &NSObject::CmdIdentify, NEEDREGNICK, 0, 0, 1 },
	{ "INFO", &NSObject::CmdInfo, 0, 0, 0, 0 },
	{ "REGISTER", &NSObject::CmdRegister, 0, 0, 0, 2 },
	{ "SET", &NSObject::CmdSet, NEEDREGNICK|NEEDFULLACCESS, 0, 0, 1 },
    	{ NULL, NULL, 0,0,0,0 }
};

struct OMessage OSmsgtab[] = {
	{ "STATUS", &OSObject::CmdStatus, NEEDOPER, 0, 0, 0 },
	{ "HASH", &OSObject::CmdHash, NEEDOPER, 0, 0 , 0},
	{ "REGHASH", &OSObject::CmdRegHash, NEEDOPER, 0, 0 , 0},
	{ "SANITY", &OSObject::CmdSanity, NEEDOPER, 0, 0 ,0},
    	{ NULL, NULL, 0,0,0,0 }
};

struct CMessage CSmsgtab[] = {
	{ "AOP", &CSObject::CmdAop, 0, 0 , 0, 1 },
	{ "REGISTER", &CSObject::CmdRegister, NEEDREGNICK|NEEDFULLACCESS, 0, 0, 1},
	{ NULL, NULL, 0,0,0,0 }
};

MessageTable::MessageTable(void)
{;}
     
MessageTable::~MessageTable(void)
{;}

void MessageTable::Check(int service, User *uptr, char *line)
{
        char    *p, *q;
        char    *args[32];
        int     c, i, argc, found;

        p = q = line;
        c = argc = i = found = 0;
        
        if(strlen(line) == 0)  
                return;

        while (*p) {
                if(*p == ' ') {
                        args[argc] = new char[512];
                        strncpy(args[argc], q, c);
			if(!argc)
				args[argc][c] = '\0';
			else
				args[argc][c-1]='\0';
                        c=0;
                        argc++;
                        q=(p+1);
                }
                p++;
                c++;
        }
        
        args[argc] = new char[512];
        sprintf(args[argc], q);
        argc++;

	switch(service) {
		case 1:						// NickServ
			while(NSmsgtab[i].cmd && !found) {
        	        	if(!strcasecmp(NSmsgtab[i].cmd, args[0])) {
					if(!this->AccessCheck(NickServ, uptr, NSmsgtab[i].flag)) {
						if(argc < (NSmsgtab[i].params+1))
						{
							uptr->Message(NickServ, RPL_NOTENOUGHPARAMS, NickServ->name, NSmsgtab[i].cmd);
							return;
						}
						(NickServ->*(NSmsgtab[i].func)) (uptr, args, argc);
					}
					found = 1;
				}
				i++;
			}
			if(!found)
				uptr->Message(NickServ, RPL_UNKNOWNCOMMAND, args[0]);
			break;
		case 2:						// OperServ
			while(OSmsgtab[i].cmd && !found) {
        	        	if(!strcasecmp(OSmsgtab[i].cmd, args[0])) {
					if(!this->AccessCheck(OperServ, uptr, OSmsgtab[i].flag)) {
						if(argc < (OSmsgtab[i].params+1)) {
							uptr->Message(OperServ, RPL_NOTENOUGHPARAMS, OperServ->name, OSmsgtab[i].cmd);
							return;
						}
						(OperServ->*(OSmsgtab[i].func)) (uptr, args, argc);
					}
					found = 1;
				}
				i++;
			}
			if(!found)
				uptr->Message(OperServ, RPL_UNKNOWNCOMMAND, args[0]);
			break;
		case 3:						// ChanServ
			while(CSmsgtab[i].cmd && !found) {
        	        	if(!strcasecmp(CSmsgtab[i].cmd, args[0])) {
					if(!this->AccessCheck(ChanServ, uptr, CSmsgtab[i].flag)) {
						if(argc < (CSmsgtab[i].params+1)) {
							uptr->Message(ChanServ, RPL_NOTENOUGHPARAMS, ChanServ->name, CSmsgtab[i].cmd);
							return;
						}
						(ChanServ->*(CSmsgtab[i].func)) (uptr, args, argc);
					}
					found = 1;
				}
				i++;
			}
			if(!found)
				uptr->Message(ChanServ, RPL_UNKNOWNCOMMAND, args[0]);
			break;
		default:
			break;
	};

        for(c=0;c<argc;c++)    
                delete args[c];
}

int MessageTable::AccessCheck(User *srptr, User *uptr, unsigned int flag) {

	aRegNick 	*nick;

	if(flag & NEEDOPER) {
		if(!IsOper(uptr)) {
			uptr->Message(srptr, RPL_NEEDOPER);
			return 1;
		}
	}

	if(flag & NEEDACCESS) {
		if(uptr->access < 1) {
			uptr->Message(srptr, RPL_NEEDACCESS);
			return 1;
		}
	}

	if(flag & NEEDFULLACCESS) {
		if(uptr->access < 2) {
			uptr->Message(srptr, RPL_NEEDFULLACCESS);
			return 1;
		}
	}

	nick = Registry->FindNick(uptr->name);

	if(flag & NEEDREGNICK) {
		if(!nick) {
			uptr->Message(srptr, RPL_NEEDREGNICK);
			return 1;
		}
	}

	return 0;
}
