/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

/* Our wrapper 
   constructors/destructors that call Init/Destroy */

NSObject::NSObject(char *name, char *username, char *hostname, 
                   Server *server, char *realname, long usermodes, time_t ts)
		: Service(name, username, hostname, server, realname, usermodes, ts)
{
	this->Help = new HelpObject("nickserv.help");
}

NSObject::~NSObject(void)
{
	delete this->Help;
}

void NSObject::Process(User *sptr, char *line)
{
	if (!line)
		return;
			
	msgtab.Check(1, sptr, line);

}	

void NSObject::Enforce(char *nick) {

	char		*newnick, *tempnick;
	aRegNick	*reg;
	Link		*f;
	User		*cptr;
	int		phase = 0;
	
	phase = atoi(nick);
	newnick = nick+1;
	tempnick = NULL;

	f = find.link(newnick);

	if(!f) {
#ifdef SDEBUG
		DebugLog->Write("Could not find the EnforceNick target: %s, aborting.", newnick);
#endif
		if(phase == 1 || phase == 2)
			FreeString(nick);
		return;
	}

	reg = Registry->FindNick(newnick);

	if(!reg) {		// Nickname is not registered, no need to be here anymore.
		if(phase == 1 || phase == 2)
			FreeString(nick);
		return;
	}

	cptr = (User *) f->value;

	if(cptr->access > 0) {	// They have identified or match the access list.
		if(phase == 1 || phase == 2)
			FreeString(nick);
		return;
	}

	switch(phase) {

		case 3:
			cptr->Message(NickServ, RPL_NICKWARN1);
			if(reg->regflags & NSF_KILL) {
				nick[0]='2';
				MakeString(tempnick, nick);
				Timers->Add(15, TIMER_ENFORCENICK, tempnick);
				cptr->Message(NickServ, RPL_NICKWARN2);
			}
			cptr->Message(NickServ, RPL_NICKWARN3);
			break;
		case 2:			
			nick[0]='1';
			ReplaceString(tempnick, nick);
			Timers->Add(15, TIMER_ENFORCENICK, tempnick);
			cptr->Message(NickServ, RPL_NICKWARN4);
			break;
		case 1:
			FreeString(nick);
			cptr->Message(NickServ, RPL_NICKWARN5);
			this->ChangeNick(cptr);
			break;
		default:	
#ifdef SDEBUG
			DebugLog->Write("Error in EnforceNick for the target %s", newnick);
#endif
			break;
	}
}

void NSObject::ChangeNick(User *cptr) {

	char	newnick[32];

	/* Let's make Services not so predictable -GZ */

	srandom(time(NULL));

	Connection.Send(":%s 433 %s %s :Belongs to another user", me->name, cptr->name, cptr->name);
	sprintf(newnick,"Guest%d",(((unsigned int)rand())%9998)+1);
	Connection.Send(":%s SVSNICK %s %s %lu", me->name, cptr->name, newnick, Timers->CTime);
}

void NSObject::Kill(User *cptr, char *message) {

	if(!IsClient(cptr))
		return;

	Connection.Send(":%s KILL %s :%s!%s (%s)", this->name, cptr->name, me->name, this->name, message);
	cptr->Delete();
}
	
void NSObject::CmdRegister(User *sptr, char **args, int argc)
{
	aRegNick	*reg;

	if((reg = Registry->FindNick(sptr->name))) {
		sptr->Message(NickServ, RPL_ALREADYREGGED, reg->name);
		return;
	}
	
	if(!IsEmail(args[2])) {
		sptr->Message(NickServ, RPL_NEEDPROPEREMAIL);
		return;
	}

	if(Email->Send(args[2], this->name,
		Translate.MakeReply(RPL_REGEMAILHEADER, sptr),
		Translate.MakeReply(RPL_REGEMAILBODY, sptr), 
		sptr->name, sptr->name, args[1])) {
			sptr->Message(NickServ, RPL_NICKREGSUCCESS, sptr->name);
			Registry->AddNick(sptr->name, sptr->username, sptr->hostname, sptr->realname);
			Registry->AddNickData1(Registry->FindNick(sptr->name), args[2], args[1], 0);
			return;
		} 
	else {
		sptr->Message(NickServ, RPL_NICKREGFAIL, args[1]);
		return;
	}		
}


void NSObject::CmdInfo(User *sptr, char **args, int argc)
{
	aRegNick	*reg;
	
	if (argc == 1)
	{
		sptr->Message(NickServ, RPL_NOTENOUGHPARAMS, NickServ->name,
			"INFO");
		return;
	}
	if ((reg = Registry->FindNick(args[1])))
	{
		sptr->Message(NickServ, RPL_INFOFOR, NickServ->name, reg->name);
	}
	else
	{
		sptr->Message(NickServ, RPL_NOTAREGNICK, args[1]);
	}

}

void NSObject::CmdHelp(User *sptr, char **args, int argc)
{
	if (argc == 1)
	{
		this->Help->Send(sptr, NickServ, "	
	}
}

void NSObject::CmdIdentify(User *sptr, char **args, int argc)
{
        aRegNick         *reg;
   
        reg = Registry->FindNick(sptr->name);

	if(!strcasecmp(args[1], reg->password)) {
		sptr->access = 2;
		sptr->Message(NickServ, RPL_PASSWORDCORRECT, sptr->name);
		return;
	}
	
	sptr->access = 0;
	sptr->Message(NickServ, RPL_PASSWORDWRONG, sptr->name);
}

void NSObject::CmdSet(User *sptr, char **args, int argc)
{
        aRegNick         *reg;
                
        reg = Registry->FindNick(sptr->name);

	if(!strcasecmp(args[1], "KILL")) {
		if(argc < 3) {
			sptr->Message(NickServ, RPL_CURRENTSETTING, "Nickname Kill", (reg->regflags & NSF_KILL) ? "ON" : "OFF");
			return;
		}
		if(!strcasecmp(args[2], "ON")) {
			reg->regflags |= NSF_KILL;
			sptr->Message(NickServ, RPL_SETTINGNOW, "Nickname Kill", "ON");
			return;
		}
		else if(!strcasecmp(args[2], "OFF")) {
			reg->regflags &= ~NSF_KILL;
			sptr->Message(NickServ, RPL_SETTINGNOW, "Nickname Kill", "OFF");
			return;
		}
		sptr->Message(NickServ, RPL_NOTENOUGHPARAMS, NickServ->name, "SET KILL");
	}
}
