/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

/* Mother of all */
Server *me;
Server *uplink = NULL;


/* Hashtable containing IRC commands, command pointers must be
    &Parse::command */
CommandTab	parsetab[] = {
	{"ERROR", &Parse::error, 15},
	{"JOIN", &Parse::join, 15},
	{"KILL", &Parse::kill, 15},
	{"MODE", &Parse::mode, 15},
	{"NICK", &Parse::nick, 15},
	{"PART", &Parse::part, 15},
	{"PING", &Parse::ping, 15},
	{"QUIT", &Parse::quit, 15},
	{"SERVER", &Parse::server, 15},
	{"SQUIT", &Parse::squit, 15},
	{"SVSKILL", &Parse::kill, 15},
	{"SVSNICK", &Parse::svsnick, 15},
	{"PRIVMSG", &Parse::privmsg, 15},
	{NULL, NULL, 0}
};

static char *para[16];

/* This parses the IRC input line */
void	Parse::parse(char *line)
{
	char 		*cmd = NULL, *s = NULL;
	CommandTab	*message = parsetab;
	int		i;
	void		*from = NULL;	
	
#ifdef SDEBUG_RECV
	DebugLog->Write("[Receiving] %s", line);
#endif
	if (*line == ':')
	{
		s = strtok(line, " ");
		s++;
		from = (void *) find.object(s);
		if (!from)
		{
			ServicesLog->Write("Unknown object %s", s);
			return;
		}
		cmd = strtok(NULL,  " ");
	}
		else
	{
		cmd = strtok(line, " ");
		if (!uplink)
			from = me;
		else
			from = uplink;
	}
	
	for (message = parsetab; message->command; message++)
		if (strcmp(message->command, cmd) == 0)
			break;

	if (message->command == NULL)
	{
		return;
	}

	if (message->func == NULL)
		return;

	i = 0;
	s = strtok(NULL, "");
	para[0] = ((User *)from)->name;
	if (s)
	{
		if (message->maxpara > 15)
			message->maxpara = 15;
		for (;;)
		{
			/*
			   ** Never "FRANCE " again!! ;-) Clean
			   ** out *all* blanks.. --msa
			 */
			while (*s == ' ')
				*s++ = '\0';

			if (*s == '\0')
				break;
			if (*s == ':')
			{
				/*
				   ** The rest is single parameter--can
				   ** include blanks also.
				 */
				para[++i] = s + 1;
				break;
			}
			para[++i] = s;
			if (i >= message->maxpara)
				break;
			for (; *s != ' ' && *s; s++)
				;
		}
	}
	para[++i] = NULL;

	(this->*(message->func)) ((void *)from, i, para);
	return;
}

void	Parse::server(void *cptr, int parc, char *parv[])
{
	Server  *tmps;
	Link	*z;

	tmps = new Server(parv[1], (Server *)cptr);	
#ifdef SDEBUG
	DebugLog->Write("Added server %s with uplink %s", parv[1],
	 	((Server *)cptr)->name);
#endif
	z = new Link(tmps, &Objects, ObjectsHash);
}

void	Parse::squit(void *cptr, int parc, char *parv[])
{
	Link *z;
	Server *p;

	z = find.link(parv[1]);
	if (!z)
	{
		ServicesLog->Write("SQUIT for unknown server %s", parv[1]);
		return;		
	}
	p = (Server *) z->value;
#ifdef SDEBUG
	DebugLog->Write("SQUITing %s", parv[1]);
	DebugLog->Write("Deleting %s from objects", parv[1]);
#endif
	z->Unlink(&Objects, ObjectsHash);
	delete p;
	delete z;
}

void	Parse::nick(void *cptr, int parc, char *parv[])
{
	User	*p, *t;
	Server  *s;
	Link	*z, *x;
	Membership *m;
	char	killnick[128];

	/* Check for nick change */

	z = find.link(parv[0]);
	if (!z)
	{
		ServicesLog->Critical("NICK: Couldn't find %s (parv[0])", parv[0]);
		me->Shutdown(1);
		exit(1);
	}
	t = (User *) z->value;
	if(IsClient(t)) {
#ifdef SDEBUG
		DebugLog->Write("Changing nick %s -> %s (%s)", parv[0], parv[1], parv[2]);
#endif
		/* We have just found a client as source, hence this is a nick change, act accordingly. */
		z->Unlink(&Objects, ObjectsHash);
		ReplaceString(t->name, parv[1]);
		z->LinkTo(&Objects, ObjectsHash);
		/* Changes TS accordingly too now */
		if (parc == 2)
		{
			t->timestamp = atol(parv[2]);
		}
		for (x = t->channels; x; x = x->next)
		{
			m = find.valueM(t,((Channel *)x->value)->members);
			if (!m)
			{
				ServicesLog->Critical("Memory data corruption, t->channels value and !t->channels->value->members");
				return;				
			}
			m->Update();
		}
	
		/* Nick change, so access is void now */

		t->access = 0;

		/* Okie - seems like everything went well, let's go for the kill-enforcement -GZ */
				
		sprintf(killnick, "3%s", t->name);	// 3 is a phase-code used later in the enforce routine.
		NickServ->Enforce(killnick);
	
		return;
	}

	/* Nope, not a nick change - must be a new client, let's add it here. */

	else if (!uplink || (cptr == uplink))
	{
		s = (Server *)find.object(parv[6]);
		if (!s)
		{
			ServicesLog->Critical("Unknown server %s in NICK from %s",
				parv[6], parv[1]);
			return;	
		}
#ifdef SDEBUG
		DebugLog->Write("Creating user %s", parv[1]);
#endif
		p = new User(parv[1], parv[4], parv[5], s, parv[8], 0, atol(parv[3]));
		z = new Link(p,&Objects, ObjectsHash);
	}

	/* Okie - seems like everything went well, let's go for the kill-enforcement -GZ */
				
	sprintf(killnick, "3%s", parv[1]);	// 3 is a phase-code used later in the enforce routine.
	NickServ->Enforce(killnick);		// See NSObject::Enforce for more info.
}

void	Parse::ping(void *cptr, int parc, char *parv[])
{
	Connection.Send("PONG :%s", MYNAME);
}

void	Parse::quit(void *cptr, int parc, char *parv[])
{
	User *p = (User *) cptr;
	Link *z, *x;
	
	if (!IsClient(p))
		return;
	
	z = find.value(cptr);
	if (!z)
	{
		return;
	}

	/* Part user ?*/
	x = p->channels;
	while (x)
	{
		parv[1] = ((Channel *)x->value)->name;
		parc = 1;
		x = x->next;
		this->part(cptr, parc, parv);
	}
#ifdef SDEBUG
	DebugLog->Write("Destroying user %s", p->name);
#endif
	z->Unlink(&Objects, ObjectsHash);
	delete p;
	delete z;
}

void	Parse::kill(void *cptr, int parc, char *parv[])
{
	Link *z;
	User *p;

	z = find.link(parv[1]);
	if (!z)
	{
		return;
	}
	p = (User *) z->value;
	if (IsMine(p))
	{
		Connection.Send(":%s KILL %s :Killing services is an absolute No-no", me->name, ((User *)cptr)->name);
		Connection.Send(":%s GLOBOPS :%s tried to kill services", me->name, ((User *)cptr)->name);
		if (IsClient(((User*)cptr)))
			this->quit(cptr, 1, parv);
		p->Spawn();
		return;
	}	
	/* Goodie, now shut down the object */
	this->quit(p, 1, parv);
}

void	Parse::svsnick(void *cptr, int parc, char *parv[])
{
	Link *z;
	User *p;

	z = find.link(parv[1]);
	if (!z)
		return;
	p = (User *) z->value;
	if (!IsClient(p))
	{
		ServicesLog->Critical("Arrrrrrgh!!! Server nick in SVSNICK command");
		return;
	}

	if (IsMine(p))
	{
		ServicesLog->Critical("Arrrrrrghhh!!!! Someone is trying to SVSNICK my client");
		return;
	}	
	/* Should we care ? */
	if (find.link(parv[2]))
	{
		ServicesLog->Critical("Arrrrrrrgh!!! Already existing nick in SVSNICK command");
		return;
	}
#ifdef SDEBUG
	DebugLog->Write("%s -> %s (SVSNICK) - timestamp %s", parv[1], parv[2], parv[3]); 
#endif
	this->nick(p, parc-1, &parv[1]);
}

void	Parse::error(void *cptr, int parc, char *parv[])
{
	ServicesLog->Write("Error from server: %s", parv[1]);
	ServicesLog->Flush();
	me->Shutdown(1);
	exit(-1);
}

void	Parse::join(void *cptr, int parc, char *parv[])
{
	User		*p = (User *) cptr;
	char 		*tmp = NULL, *channel = NULL, *o = NULL;
	Link		*z = NULL;
	Channel 	*c = NULL;
	Membership	*y = NULL;
	
	for (tmp = parv[1]; (channel = strtoken(&o, tmp, ",")); tmp = NULL)

	{
		/* We need JOIN 0 coding too */
		if (!strcmp(channel, "0"))
		{
			z = p->channels;
			while (z)
			{
				/* We'd want to preserve parv[1] */
				parv[2] = p->name;
				parv[3] = ((Channel *)z->value)->name;
				parc = 1;
				z = z->next;
				this->part(cptr, parc, &parv[2]);
			}

			continue;
		}

		z = find.link(channel);
		
		if (!z)
		{
			/* Channel doesn't exist - make it */
			c = new Channel(channel);
			z = new Link(c, &Objects, ObjectsHash);
#ifdef SDEBUG
			DebugLog->Write("Creating channel %s", channel); 
#endif
		}
		else
		{
			c = (Channel *) z->value;
		}
#ifdef SDEBUG
			DebugLog->Write("Joining %s into %s", p->name, channel);
#endif
		y = new Membership(0L, p);
		z = new Link(y, &c->members);
		z = new Link(c, &p->channels);

		/* User joins a channel, let's inform ChanServ about it. -GZ */

		ChanServ->JoinUser(p, c);
	}
}


void	Parse::part(void *cptr, int parc, char *parv[])
{
	User		*p = (User *) cptr;
	char 		*tmp = NULL, *channel = NULL, *o = NULL;
	Link		*z = NULL, *x = NULL;
	Channel 	*c = NULL;
	Link		*y = NULL;
	
	for (tmp = parv[1]; (channel = strtoken(&o, tmp, ",")); tmp = NULL)
	{
		x = (Link *)find.link(channel);
		if (!x)
		{
#ifdef SDEBUG
			DebugLog->Write("PART for unknown channel %s from user %s", channel, p->name);
#endif
			continue;
			
		}
		c = (Channel *) x->value;
		if (!IsChannel(c))
		{
#ifdef SDEBUG
			DebugLog->Write("PART: find.object returned !IsChannel for channel %s", channel);
#endif
			continue;
		}
		z = find.link(p->name, c->members);
		if (!z)
		{
#ifdef SDEBUG
			DebugLog->Write("PART for user %s which is not in %s member list", p->name, channel);
#endif
			continue;
		}
		/* Let's start the parting */
		y = find.link(c->name, p->channels);
		y->Unlink(&p->channels);
		delete y;
#ifdef SDEBUG
		DebugLog->Write("PARTing user %s from %s", p->name, c->name);
#endif
		z->Unlink(&c->members);
		delete z;
		
		if (!c->members)
		{
#ifdef SDEBUG
			DebugLog->Write("Destroying channel %s", c->name);
#endif
			x->Unlink(&Objects, ObjectsHash);
			delete x;
			delete c;
		}

	}
}


void	Parse::mode(void *cptr, int parc, char *parv[])
{
	User *uptr = (User *) cptr;
	if (*parv[1] == '#')
	{
		/* Launch # mode stuff here */
		return;
	}	
	if (!IsClient(uptr))
	{
#ifdef SDEBUG
		DebugLog->Write("Non-client %s trying to set mode on itself",
			uptr->name);	
#endif
		return;
	}		
	uptr->SetMode(parv[2]);
#ifdef SDEBUG
	DebugLog->Write("Set mode %s on %s (final=%s)", parv[2], uptr->name,
		(char *)uptr->GetMode());
#endif
}


void	Parse::kick(void *cptr, int parc, char *parv[])
{
#ifdef SDEBUG
	User *uptr = (User *) cptr;
#endif
	Channel *chptr = NULL;	
	Membership *mptr = NULL;
	User *tptr = NULL;
	
	chptr = (Channel *)find.object(parv[1]);
	if (!chptr)
	{	
#ifdef SDEBUG
		DebugLog->Write("KICK (%s -> %s (%s)) for unknown channel %s",
					parv[0], parv[2], parv[3], parv[1]);
#endif
		return;
	}	
	mptr = (Membership *)find.object(parv[2], chptr->members);
	if (!mptr)
	{
#ifdef SDEBUG
		DebugLog->Write("KICK (%s -> %s (%s)) for unknown nick at %s",
					parv[0], parv[2], parv[3], parv[1]);
#endif
		return;
	}
	tptr = mptr->value;
	parv[0] = tptr->name;
	parc = 1;
#ifdef SDEBUG
	DebugLog->Write("Kicked %s from channel %s (kicked by %s) reason: %s",
				tptr->name, chptr->name, uptr->name, parv[3]);
#endif
	this->part((void*) tptr, parc, parv);
	return;
}


void	Parse::privmsg(void *cptr, int parc, char *parv[])
{
	User *uptr = (User *) cptr;
	User *acptr = NULL;
	Service *sptr = NULL;
	if (!IsClient(uptr))
	{
#ifdef SDEBUG
		DebugLog->Write("Got PRIVMSG from server %s", uptr->name);
#endif
		return;
	}
	if (*parv[1] == '#')
		return;	
	
	acptr = (User *)find.object(parv[1]);
	if (!acptr)
	{
#ifdef SDEBUG
		DebugLog->Write("Couldn't find user %s in PRIVMSG from %s",
				parv[1], uptr->name);
#endif
		return;
	}
	
	if (!IsMine(acptr))
	{
#ifdef SDEBUG
		DebugLog->Write("Got !Mine user %s in PRIVMSG from %s",
				parv[1], uptr->name);		
#endif
		return;
	}
	if (IsMine(acptr))
	{
#ifdef SDEBUG
		DebugLog->Write("Message [%s -> %s]: %s",
				uptr->name, parv[1], parv[2]);	
#endif
		sptr = (Service *) acptr;
		sptr->Process(uptr, parv[2]);
	}
	return;	
}
