/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

Reg::Reg(void)
{;}
       
Reg::~Reg(void)
{;}
   
aRegNick *Reg::FindNick(char *nick)
{
        Link    *p;
        long    hash;

        hash = get_hash_value(nick);

        for (p = RegdataHash[hash]; p; p = p->hnext)
                if (!strcasecmp(((RegLink*) p->value)->name, nick))
                {
			if(IsClient(((RegLink*) p->value)))
	                        return(((RegLink*) p->value)->nick);
                }

        return NULL;	
}

aRegChan *Reg::FindChannel(char *channel)
{
        Link    *p;
        long    hash;

        hash = get_hash_value(channel);

        for (p = RegdataHash[hash]; p; p = p->hnext)
                if (!strcasecmp(((RegLink*) p->value)->name, channel))
                {
			if(IsChannel(((RegLink*) p->value)))
	                        return(((RegLink*) p->value)->chan);
                }

        return NULL;	
}

void Reg::AddChannel(char *channel, char *founder){

	RegLink	*newreg;
	Link	*reglink;

	newreg = new RegLink(channel, FLAGS_CHANNEL, founder);
	
        reglink = new Link(newreg);
        
        reglink->LinkRegTo(&Regdata, RegdataHash);
}
  
void Reg::AddChannelData1(aRegChan *chan, long flags)
{
	chan->regflags = flags;
}

aChanOp *Reg::FindChannelOp(aRegChan *chan, char *nick) {

	Link	*f;
	aChanOp	*op;

        for(f = chan->oplist;f;f = f->next) {
                op = (aChanOp *) f->value;
                        if(!strcasecmp(op->name, nick)) 
                        	return op;
        }
        
        return NULL;
}

int  Reg::GetOpLevel(aRegChan *chan, char *nick) {

	Link	*f;
	aChanOp *op;

	if(!chan->oplist)
		return 0;

       	for(f = chan->oplist;f;f = f->next) {
        	op = (aChanOp *) f->value;
                	if(!strcasecmp(op->name, nick))
                        	return op->level;
	}

	return 0;
}

void Reg::AddChannelOp(aRegChan *chan, aRegNick *nick, long flags)
{
	aChanOp	*chanop = NULL;		
	Link	*p = NULL, *n = NULL;

	p = chan->oplist;

	chanop = new aChanOp;
	MakeString(chanop->name, nick->name);
	chanop->level = flags;

	if(!p) {
		p = new Link(chanop);
		chan->oplist = p;
	}
	else {
		n = new Link(chanop);
		n->LinkTo(&chan->oplist);
	}
}

void Reg::AddNick(char *nick, char *username, char *hostname, char *realname)
{
        RegLink       *newreg;
        Link           *reglink;
    
        newreg = new RegLink(nick, FLAGS_CLIENT,  username, hostname, realname);

        reglink = new Link(newreg);
   
        reglink->LinkRegTo(&Regdata, RegdataHash);
}

void Reg::AddNickData1(aRegNick *nick, char *email, char *password, long flags)
{
	MakeString(nick->email, email);
	MakeString(nick->password, password);
	nick->regflags = flags;

	return;
}

void Reg::Delete(char *nick)
{
        Link    *p;
        long    hash;

        hash = get_hash_value(nick);

        for (p = RegdataHash[hash]; p; p = p->hnext)
                if (!strcasecmp(((RegLink*) p->value)->name, nick))
                {
                        delete (RegLink*) p->value;
                        p->RegUnlink(&Regdata, RegdataHash);
                        p->Destruct();
                        delete p;
			return;
                }
#ifdef SDEBUG
	DebugLog->Write("Could not delete the nick %s from the registry!", nick);
#endif
}

RegLink::RegLink(char *channel, long flags, char *founder)
{
	MakeString(this->name, channel);
	this->chan = new aRegChan;
	MakeString(this->chan->name, channel);
	MakeString(this->chan->founder, founder);
	this->chan->oplist = NULL;
	this->flags = flags;
}

RegLink::RegLink(char *nick, long flags, char *username, char *hostname, char *realname)
{
	MakeString(this->name, nick);
	this->nick = new aRegNick;

	MakeString(this->nick->name, nick);
	MakeString(this->nick->username, username);
	MakeString(this->nick->hostname, hostname);
	MakeString(this->nick->realname, realname);
	this->nick->activate = 0;
	this->nick->email = NULL;
	this->nick->password = NULL;
	this->nick->regflags = 0;
	this->flags = flags;
}
       
RegLink::~RegLink(void)		
{
	if(IsChannel(this)) {
                if (this->name)
                        FreeString(this->name);
		if (this->chan) {
	                if (this->chan->name)      
        	                FreeString(this->chan->name);  			
	                if (this->chan->founder)      
        	                FreeString(this->chan->name);  			
			if (this->chan->oplist)
				delete this->chan->oplist;
			delete this->chan;
		}
		this->flags = 0;
	}
	else if (IsClient(this)) {
		if (this->name)
			FreeString(this->name);
		if (this->nick) {
			if (this->nick->name)
				FreeString(this->nick->name);
			if (this->nick->username)
				FreeString(this->nick->username);
			if (this->nick->hostname)
				FreeString(this->nick->hostname);
			if (this->nick->realname)
				FreeString(this->nick->realname);
			if (this->nick->email)
				FreeString(this->nick->email);
			if (this->nick->password)
				FreeString(this->nick->password);
			this->nick->regflags = 0;
			this->nick->activate = 0;
			delete	this->nick;
		}
		this->flags = 0;
	}
	else {
#ifdef SDEBUG
		DebugLog->Write("UNKNOWN REGLINK DESTRUCTION: Object named %s", this->name);
#endif
	}
}
