/*
   Sirius IRC Services 2 - socket.cpp
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

/* This connects the socket to a DNS/IP <server> at TCP port <port> */
int	Socket::Connect(char *server, int port)
{
	struct	sockaddr_in	sa;
	struct	hostent		*hp;
	int			s;

	if ((hp = this->Resolve(server)) == NULL)
	{
		errno = ECONNREFUSED;
		return (-1);	
	}
	bzero(&sa, sizeof(sa));
	// Set address
	bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);	
	sa.sin_family = hp->h_addrtype;
	sa.sin_port = htons((u_short) port);
	
	if ((s = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0)
	{
		this->fd = -1;
		return(-1);
	}
	if (connect(s, (struct sockaddr *) &sa, sizeof(sa)) < 0)
	{
		close(s);
		this->fd = -1;
		return (-1);
	}
	this->fd = s;
	return (s);
}

/*
 * The following functions are general DNS tools
*/
struct hostent *Socket::Resolve (char *hostname)
{
	struct hostent *hp;
	
	if ((hp = this->lookupHost(hostname)) == NULL)
		hp = this->lookupIP(hostname);

	return hp;
}

struct hostent *Socket::lookupHost(char *host)
{
	struct hostent *hp;

	alarm(5);	
	hp = gethostbyname(host);
	alarm(0);
	return hp;
}

struct hostent *Socket::lookupIP(char *ip)
{
	int p1,p2,p3,p4;
	char doot[4];
	struct hostent *hp;
	
	p1=p2=p3=p4=0;
	
	sscanf(ip, "%i.%i.%i.%i", &p1, &p2, &p3, &p4);
	doot[0] = p1;
	doot[1] = p2;
	doot[2] = p3;
	doot[3] = p4;
	alarm(5);
	hp = gethostbyaddr(doot, 4, AF_INET);
	alarm(0);
	return hp;
}

/* General Send to socket 
  
   Same format as printf
 
   Example:
	Socket::Send(":%s NOTICE %s :moo", ME, User->nick);
*/

void	Socket::Send(char *format, ...)
{
	static char	Buffer[513];
	unsigned short	bufferLen;
	va_list		ap;
	int		error;

	va_start(ap, format);
	vsnprintf(Buffer, 511, format, ap);

	if(!ServicesConnect) {
#ifdef SDEBUG
		DebugLog->Write("Unable to send: %s", Buffer);
#endif
		return;
	}
#ifdef SDEBUG_SEND 
        DebugLog->Write("[Sending] %s", Buffer);
#endif
	strcat(Buffer, "\n");
	bufferLen = strlen(Buffer);
	error = write(this->fd, Buffer, bufferLen);
	if ((error < 1) && (errno != 0))
	{
		if (!(this == &Connection))
		{
			ServicesLog->Write("Socket::Send() error in socket %i: %s",
				strerror(errno), this->fd);  
		}
			else
		{
			ServicesLog->Write("Socket::Send() error at Connection: %s - Shutting down", strerror(errno));
			me->Shutdown(1);
			exit(0);
		}		
	}
	return;
}

