/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
   
#include "services.h"

/* This is the Timer object which will be the object that will be used to control
 * the timers itself. 
 *
 * Timer->Check 	- Execute any timers that are due.
 * Timer->Add 		- Add a new timer to the timer list.
 * Timer->Delete	- Remove a timer with a specific ID from the list. 
 *
 */
Timer::Timer(void)
{
	this->CTime = time(NULL);
	this->lastcheck = this->CTime;
}
     
Timer::~Timer(void)
{
	this->CTime = 0;
	this->lastcheck = 0;
}

/*
 * TimerLink is the actual timer itself, containing the functions, args etc.
 */

TimerLink::TimerLink(long time, int timerid, char *args)
{
	this->Expire = (Timers->CTime+time);
	this->id = timerid;
	this->args = args;
	this->Tag = Timers->total++;
}
     
TimerLink::~TimerLink(void)
{
	this->Expire = 0;
	this->Tag = 0;
	this->id = 0;
	this->args = NULL;
}

/* Timer Control functions (Timer Class) */

void Timer::Check(void)
{
	Link *c;
	long i = 0;
	TimerLink *wipe;

	if(!(time(NULL) > this->lastcheck))
		return;
	
	this->lastcheck = this->CTime = time(NULL);

        for (c = TimerList; c; c = c->next) {
		i = ((TimerLink*) c->value)->Expire - Timers->CTime;
		if(i <= 0) {
		        wipe = ((TimerLink*) c->value);
			TimerWrapper(wipe->id, wipe->args);
        		delete wipe;   
        		c->Unlink(&TimerList);
		        c->Destruct();
			delete c;	
		}
	}
}

void Timer::Add(long time, int timerid, char *args) {

	TimerLink	*newtimer;
	Link		*timerlink;

	if(time <= 0)
		TimerWrapper(timerid, args);

	newtimer = new TimerLink(time, timerid, args);	
	timerlink = new Link(newtimer);

	timerlink->LinkTo(&TimerList);
}

void Timer::Delete(int Tag) {

       	Link *c;
        TimerLink *wipe;
        
        for (c = TimerList; c; c = c->next) {
        	wipe = ((TimerLink*) c->value);
		if(wipe->Tag == Tag) {
                        delete wipe;
                        c->Unlink(&TimerList);
                        c->Destruct();   
                        delete c;
                }
        }
}

/* This is a wrapper function to allow the timer to call C++ class functions
 * definitions are to be made in the timer.h file
 */

void TimerWrapper(int function, char *args) {

	switch(function) {
#ifdef SDEBUG
		case TIMER_DEBUGLOG:
			DebugLog->Write("%s", args);
			break;
#endif
		case TIMER_ENFORCENICK:
			NickServ->Enforce(args);
			break;
		default:
			break;
	}	
}
