/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"
#include "modetab.h"

/* Our constructor
   ----------------
	name = nick
        username, hostname = user@host
        Server = orgining server object
	realname = real name field
	usermodes = usermodes of the user
	ts = IRC TS of the user
*/

User::User(char *name, char *username, char *hostname, 
                   Server *server, char *realname, long usermodes, time_t ts)
{
	this->flags = FLAGS_CLIENT;

	MakeString(this->name, name);
	MakeString(this->username, username);
	MakeString(this->hostname, hostname);
	MakeString(this->realname, realname);
	this->server = server;
	this->usermodes = usermodes;
	this->timestamp = timestamp;	
	this->channels = NULL;
	this->language = Translate.SetLanguage(this);
	this->access = 0;
	this->reg = NULL;
}

/*
 * Our destructor:
 * ---------------
 * This cleans out the object and free's the variables and
 * resets them to NULL
*/

User::~User(void)
{
	if (this->name)
		FreeString(this->name);
	if (this->username)
		FreeString(this->username);
	if (this->hostname)
		FreeString(this->hostname);
	if (this->realname)
		FreeString(this->realname);
	/* We'll want to clean out properly */
	this->name = this->username = this->hostname = this->realname = NULL;
	this->flags = 0;
	this->server = NULL;
	this->usermodes = 0;
	this->timestamp = 0;
	this->language = 0;
	this->channels = NULL;
	this->language = 0;
	this->access = 0;
	this->reg = NULL;
}

int User::Spawn(void)
{
	if(!(this->flags & FLAGS_MINE))
	{
		ServicesLog->Write("Spawn for non local user %s", this->name);
		return 0;
	}
	else {
		Connection.Send("NICK %s 1 %li %s %s %s :%s", this->name,
			this->timestamp, this->username, this->hostname,
			this->server->name, this->realname);
			
		Connection.Send(":%s MODE %s %s", this->name, this->name, 
			(char *) this->GetMode());
		return 1;
	}
}

/* Code originally by Dave Dunn of the Mercury project */
void	User::SetMode(char *string)
{
	char 	*p;
	Umode 	*x;
	int  	mode = 1;		/* 1 = add 0 = remove */

	for (p = string; *p; p++)
	{
		if (*p == '+')
		{
			mode = 1;
			continue;
		}
		if (*p == '-')
		{
			mode = 0;
			continue;
		}
		if (*p == ' ')
			return;
		for (x = umodetab; !(x->flag == 0); x++)
		{
			if (*p == x->mode)
			{
				if (mode == 1)
					this->usermodes |= x->flag;
				if (mode == 0)
					this->usermodes &= ~x->flag;
			}
		}
	}
}

void	User::SetMode(long value, int mode)
{
	if (mode == 1)
		this->usermodes |= value;
	if (mode == 0)
		this->usermodes &= ~value;
}


char	*User::GetMode(void)
{
	static char	modebuf[100];
	Umode		*p;
	char		*m = modebuf;
	
	p = umodetab;
	*m++ = '+';
	while (p->flag != 0)
	{
		if (this->usermodes & p->flag)
		{
			*m++ = p->mode;
		}
		p++;
	}
	*m = '\0';
	return (modebuf);
}

void	User::Message(User *from, char *pattern, ...)
{
	va_list	ap;
	char	buff[1024], buf[1024];

	va_start(ap, pattern);
	snprintf(buff, 1020, ":%s NOTICE %s :%s", from->name, this->name,
		pattern);
	vsnprintf(buf, 1020, buff, ap);
	va_end(ap);
	Connection.Send("%s", buf);
}

void	User::Message(User *from, int numeric, ...)
{
	va_list	ap;
	char	buff[1024], buf[1024];

	va_start(ap, numeric);
	snprintf(buff, 1020, ":%s NOTICE %s :%s", from->name, this->name,
		Translate.MakeReply(numeric, this));
	vsnprintf(buf, 1020, buff, ap);
	va_end(ap);
	Connection.Send("%s", buf);
}

Service::Service(char *name, char *username, char *hostname, Server *server, char *realname,
		     long usermodes, time_t ts) : User(name, username, hostname,
                     server, realname, usermodes,ts)
{
}

Service::~Service()
{
}

void Service::Process(User *sptr, char *line) {};

void User::PartChannel(char *channel) {

        User            *p = (User *) this;
        Link            *z = NULL, *x = NULL;
        Channel         *c = NULL;
        Link            *y = NULL;

        x = (Link *)find.link(channel);

        if (!x) {
#ifdef SDEBUG
        	DebugLog->Write("PART for unknown channel %s from user %s", channel, p->name);
#endif
               	return;
        }

        c = (Channel *) x->value;

        if (!IsChannel(c)) {
#ifdef SDEBUG
        	DebugLog->Write("PART: find.object returned !IsChannel for channel %s", channel);
#endif
                return;
        }

        /* Let's start the parting */

        y = find.link(c->name, p->channels);
        y->Unlink(&p->channels);
        delete y;

        z = find.link(p->name, c->members);

        if (!z) {
#ifdef SDEBUG
        	DebugLog->Write("PART for user %s which is not in %s member list", p->name, channel);
#endif   
                return;
        }

#ifdef SDEBUG
	DebugLog->Write("Parting %s from channel %s", this->name, c->name);
#endif	

        z->Unlink(&c->members);
        delete z;

        if (!c->members) {
#ifdef SDEBUG
        	DebugLog->Write("Destroying channel %s", c->name);
#endif
                x->Unlink(&Objects, ObjectsHash);
                delete x;
                delete c;
      	}
}

void User::PartAllChannels(void) {

        User            *p = (User *) this;
        Link            *z = NULL, *y = NULL;

        z = p->channels;

        while (z) {
		y = z->next;
                this->PartChannel(((Channel *)z->value)->name);
        	z = y;
        }
}

void User::Delete(void) {
        User *p = (User *) this;
        Link *z;

        if (!IsClient(p))
                return;
                
        z = find.value(this);
        if (!z)
        {
                return;
        }

	/* Remove from all channels */
        
	this->PartAllChannels();

#ifdef SDEBUG
        DebugLog->Write("Destroying user %s", p->name);
#endif
        z->Unlink(&Objects, ObjectsHash);
        delete p;
        delete z;
}
